﻿Type=Service
Version=3.82
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	Private manager As WifiManager
	Private discoveredDevices As List
	Private server As ServerSocket
	Private client As Socket
	Private port As Int = 13138
	Private astream As AsyncStreams
	Private directConnected As Boolean
	Private groupIp As String
	Private countingStream As CountingInputStream
	Private totalSizeForSending As Long
	Private currentFile As String
	Private timer1 As Timer
	Public progressValue As Int
	Public progressText, lblFile As String
	Public SendingFile As Boolean
	Public ReceivingFile As Boolean
	Public lblEnabledText, lblConnectedText, lblDevicesText As String
	Public btnChooseFileEnabled = False, btnConnectEnabled = True As Boolean
End Sub

Sub Service_Create
	server.Initialize(port, "server")
	manager.Initialize("manager")
	timer1.Initialize("timer1", 1000)
End Sub

Sub Manager_EnabledChanged (Enabled As Boolean)
	Log("Enabled: " & Enabled)
	lblEnabledText = Enabled
	UpdateUI
End Sub

Public Sub Connect
	lblConnectedText = "Trying to connect..."
	If directConnected = False Then
		'create the p2p connection
		If discoveredDevices.IsInitialized = False OR discoveredDevices.Size = 0 Then 
			ToastMessageShow("No peered devices.", True)
			Return
		End If
		Dim dev As WifiDevice = discoveredDevices.Get(0)
		manager.Connect(dev.MacAddress)
	Else
		If astream.IsInitialized Then Return 'there is already a network connection
		'there is already a p2p connection
		ConnectNetwork
	End If
	UpdateUI
End Sub

Public Sub Discover
	lblDevicesText = "Searching for devices"
	manager.DiscoverPeers
	UpdateUI
End Sub

Sub Manager_PeersDiscovered (Success As Boolean, Devices As List)
	Log("peers: " & Success)
	If Success Then
		discoveredDevices = Devices
		lblDevicesText = Devices.Size
	Else
		lblDevicesText = "0"
	End If
	UpdateUI
End Sub

Public Sub SendFile(Dir As String, FileName As String)
	Dim totalSizeForSending As Long = File.Size(Dir, FileName)
	If totalSizeForSending = 0 Then
		Log("Error getting file size.")
		Return
	End If
	Dim In As InputStream = File.OpenInput(Dir, FileName)
	countingStream.Initialize(In)
	currentFile = FileName.SubString(FileName.LastIndexOf("/") + 1)
	astream.Write(currentFile.GetBytes("UTF8")) 'write the file name
	astream.WriteStream(countingStream, totalSizeForSending)
	lblFile = "Sending: " & currentFile
	timer1.Enabled = True
	SendingFile = True
	UpdateProgress
End Sub

Sub Manager_ConnectionChanged (Connected As Boolean, GroupOwnerIp As String)
	directConnected = Connected
	groupIp = GroupOwnerIp
	If Connected Then
		lblConnectedText = "Wifi Direct connected (" & GroupOwnerIp & ")"
		ConnectNetwork
	Else
		lblConnectedText = "false"
	End If
	btnConnectEnabled = Not(Connected)
	UpdateUI
End Sub

Sub ConnectNetwork
	If groupIp = "127.0.0.1" Then
		server.Listen
	Else
		Log("trying to connect to server socket.")
		client.Initialize("client")
		client.Connect(groupIp, port, 10000)
	End If
End Sub

Sub client_Connected (Successful As Boolean)
	If Successful Then
		directConnected = True
		StartAsyncStreams(client)
	Else
		lblConnectedText = "client failed to connect."
		btnConnectEnabled = True
		UpdateUI
	End If
End Sub

Sub server_NewConnection (Successful As Boolean, NewSocket As Socket)
	Log("server_NewConnection " & Successful)
	If Successful Then
		directConnected = True
		StartAsyncStreams(NewSocket)
	End If
	server.Listen
End Sub

Private Sub UpdateUI
	CallSub(Main, "UpdateUI")
End Sub

Private Sub UpdateProgress
	CallSub(Main, "UpdateProgress")
End Sub

Public Sub Disconnect
	If directConnected = True Then
		astream.Close
		astreams_Terminated
	End If
End Sub

Sub StartAsyncStreams(s As Socket)
	Log("AsyncStreams started")
	astream.InitializePrefix(s.InputStream, True,  s.OutputStream, "astreams")
	Dim TargetSDcard As String : TargetSDcard = "/mnt/extSdCard"
	Dim SubDir As String : SubDir = "WDfiletransfer"
	If File.IsDirectory(TargetSDcard, "") = True Then 
		TargetSDcard = TargetSDcard
	Else
		If File.ExternalWritable Then 
			TargetSDcard = File.DirRootExternal
		Else
			Msgbox("Application will now close ...","Storage space unavailable")
			Disconnect
			ExitApplication
		End If
	End If
	If File.IsDirectory(TargetSDcard, SubDir) = False Then
		File.MakeDir(TargetSDcard, SubDir)
	End If
	astream.StreamFolder = TargetSDcard & "/" & SubDir
	btnChooseFileEnabled = True
	btnConnectEnabled = False
	lblConnectedText = "Sockets are connected :-)"
	UpdateUI
End Sub

Sub astreams_Error
	Log("astreams_Error")
	ToastMessageShow(LastException, True)
	Log(LastException)
	astream.Close
	astreams_Terminated
End Sub

Sub astreams_Terminated
	Log("astreams_Terminated")
	btnChooseFileEnabled = False
	btnConnectEnabled = True
	lblConnectedText = "Sockets disconnected"
	UpdateUI
End Sub

Sub astreams_NewStream (Dir As String, FileName As String)
	'this event is raised when a file was received successfully
	Timer1_Tick
	timer1.Enabled = False
	lblFile = currentFile & " completed"
	ReceivingFile = False
	UpdateProgress
	File.Copy(Dir, FileName, Dir, currentFile)
	File.Delete(Dir, FileName)
End Sub

Sub astreams_NewData (Buffer() As Byte)
	'get the file name
	timer1.Enabled = True
	currentFile = BytesToString(Buffer, 0, Buffer.Length, "UTF8")
	lblFile = "Receiving file: " & currentFile
	ReceivingFile = True
End Sub

Sub Timer1_Tick
	Dim count, total As Long
	If SendingFile Then
		count = countingStream.count
		total = totalSizeForSending
		If count = total Then
			lblFile = currentFile & " completed"
			'stop the timer.
			'when a file is received the NewStream event will be raised
			SendingFile = False
			timer1.Enabled = False
		End If
	Else If ReceivingFile Then
		count = astream.StreamReceived
		total = astream.StreamTotal
	End If
	progressValue = 100 * count / total
	progressText = NumberFormat2(count / 1000, 0, 0, 0, True) & _
			"kb / " & NumberFormat2(total / 1000, 0, 0, 0, True) & "kb"
	UpdateProgress		
End Sub

Sub Service_Start (StartingIntent As Intent)

End Sub

Sub Service_Destroy

End Sub